#ifndef __MORE_ATOM_TYPES_H__
#define __MORE_ATOM_TYPES_H__

#include "pdb.h"
#ifdef __cplusplus
extern "C" {
#endif

/*
   Use the following two functions to change the radii 
   used and to print out the current radii. 
   You can use RICHARDS_RADII or CHC_RADII 
   
    WriteAtomTypes();
    UpdateAtomTypes(stream_from_string (NUCPROT_RADII)) ;
    WriteAtomTypes();

    To Compile:
   More_atom_types.o:
	cc $(CFLAGS) -Wf,-XNl10000 -c More_atom_types.o More_atom_types.c
*/

void WriteAtomTypes(void)
;
void UpdateAtomTypes(FILE * ff)
;

typedef struct char8 { char name[8] ; } char8 ;

extern double atom_vdw[ MAX_ATOM_TYPE ] ;
extern double atom_covalent[ MAX_ATOM_TYPE ] ;
extern double atom_charge[ MAX_ATOM_TYPE ] ;
extern char8 atom_type_names[ MAX_ATOM_TYPE ] ;

/* ATOM_TYPE VDW_RADIUS COVALENT_RADIUS CHARGE */

#define RICHARDS_RADII "\
C3H0s	1.70	0.77\n\
C3H0b	1.70	0.77\n\
C3H1t	1.85	0.77\n\
C3H1s	1.85	0.77\n\
C3H1b	1.85	0.77\n\
C3H2u	1.85	0.77\n\
C4H1s	2.00	0.77\n\
C4H1b	2.00	0.77\n\
C4H2s	2.00	0.77\n\
C4H2b	2.00	0.77\n\
C4H3u	2.00	0.77\n\
N2H0s	1.50	0.70\n\
N2H0b	1.50	0.70\n\
N3H0u	1.50	0.70\n\
N3H1s	1.70	0.70\n\
N3H1b	1.70	0.70\n\
N3H2u	1.80	0.70\n\
N4H3u	2.00	0.70\n\
O1H0u	1.40	0.66\n\
O2H0s	1.40	0.66\n\
O2H0b	1.40	0.66\n\
O2H1u	1.60	0.66\n\
O2H2u	1.60	0.66\n\
S2H0u	1.85	1.04\n\
S2H1u	2.00	1.04\n\
P4H0u	0.00	1.10\n\
O1N3HH	1.60	0.68\n\
Z2ION	0.74	0.74\n\
FE	1.70	0.70\n\
DEFAULT	1.80	0.77\n\
"


#define RICHARDS_RADII_OLD "\
C4	0.00  0.77\n\
C4H	2.00  0.77\n\
C4HH	2.00  0.77\n\
CPRO	2.00  0.77\n\
C4HHH	2.00  0.77\n\
C3	1.70  0.77\n\
C3H	1.85  0.77\n\
C3HH	1.85  0.77\n\
OWAT    1.40  0.77\n\
OW      1.40  0.77\n\
O1	1.40  0.66\n\
O2H	1.60  0.66\n\
O1O2H	1.50  0.66\n\
N4	0.00  0.70\n\
N4H	2.00  0.70\n\
N4HH	2.00  0.70\n\
N4HHH	2.00  0.70\n\
N3	1.50  0.70\n\
N3H	1.70  0.70\n\
N3HH	1.80  0.70\n\
S2	1.85  1.04\n\
S2H	2.00  1.04\n\
P4	0.00  1.10\n\
O1N3HH	1.60  0.68\n\
Z2ION	0.74  0.74\n\
FE	1.70  0.70\n\
DEFAULT	1.80  0.77\n\
"


#define CHC_RADII "\
C4      0.00    0.77\n\
C4H     1.87    0.77\n\
C4HH    1.87    0.77\n\
CPRO    1.87    0.77\n\
C4HHH   1.87    0.77\n\
C3      1.76    0.77\n\
C3H     1.76    0.77\n\
C3HH    0.0     0.77\n\
OWAT    1.40    0.77\n\
OW      1.40    0.77\n\
O1      1.40    0.66\n\
O2H     1.40    0.66\n\
O1O2H   1.40    0.66\n\
N4      0.00    0.70\n\
N4H     1.50    0.70\n\
N4HH    1.50    0.70\n\
N4HHH   1.50    0.70\n\
N3      1.5     0.70\n\
N3H     1.65    0.70\n\
N3HH    1.65    0.70\n\
S2      1.85    1.04\n\
S2H     1.85    1.04\n\
P4      0.00    1.10\n\
O1N3HH  1.60    0.68\n\
Z2ION   0.74    0.74\n\
FE      1.70    0.70\n\
DEFAULT	1.80  0.77\n\
"

#ifdef __cplusplus
}
#endif

#endif /* !__MORE_ATOM_TYPES_H__ */

